﻿<#
        .SCRIPTNAME
        AppRegManager2024_v03.ps1 - info@microcloud.nl
		
		.REVISIONS
    2024_v03 - Save password (cleartext) in export file? (Y/N) -> YES was not working. Fixed.
    2024_v02 - Update, Object ID instead of AppID was exported. Fixed.
		2024_v01 - first release
       
        .SYNOPSIS
        Create, manage and report Entra ID Application Registration/Service Principal Name

        .DESCRIPTION
        Create and manage Entra ID Application Registration/Service Principal Name. Script offers several features:
          -> Create new AppReg/SPN with a new certificate
          -> Add a new certificate to an existing AppReg/SPN
          -> Delete AppReg/SPN from Entra ID
          -> Check and report expiration status of secrets and certificates
          -> Check and report Graph and Entra ID Role permissions for AppReg/SPN
         
        https://www.microcloud.nl
#>

#############################################################################################################################
$global:ScriptName           = $MyInvocation.MyCommand.Name                              # current script name
$global:ScriptXMLName        = $global:ScriptName.Replace(".ps1",".xml")                 # XML filename
$global:ScriptNameAndPath    = $MyInvocation.MyCommand.Definition                        # put full file and pathname in var.
$global:ScriptPath           = $global:ScriptNameAndPath.Replace($global:ScriptName, "") # current script path
$global:ScriptXMLNameAndPath = $global:ScriptPath+$global:ScriptXMLName 
$global:timestamp            = (Get-Date).ToString("yyyyMMdd-HHmm") 
#############################################################################################################################

$global:defaultpwd="P@ssw0rd"
$global:disconnect=$true # If set to true, after each action the script disconnects from Microsoft Graph, require to login every time again.
$expirewarning=60 # 60 days before credential expiration give a warning!

Function Remove-InvalidFileNameChars 
{
  # Source: https://stackoverflow.com/questions/23066783/how-to-strip-illegal-characters-before-trying-to-save-filenames
  param(
    [Parameter(Mandatory=$true,
      Position=0,
      ValueFromPipeline=$true,
      ValueFromPipelineByPropertyName=$true)]
    [String]$Name
  )
  $invalidChars = [IO.Path]::GetInvalidFileNameChars() -join ''
  $re = "[{0}]" -f [RegEx]::Escape($invalidChars)
  return ($Name -replace $re)
}

Function ExitScript
  {
    If ($global:disconnect) 
      { 
         Write-host "Disconnecting from Microsoft Graph ..." -NoNewline
         disconnect-MgGraph -ErrorAction SilentlyContinue
         start-sleep 1
         Write-host " done!"
      }    
    exit 
  }

Function Option1AskQ ($option)
  {
    #     
    $AZTenant=Get-MgOrganization
    $AZtenantname=Remove-InvalidFileNameChars($($AZTenant.DisplayName))
    $TenantName=($AZTenant.VerifiedDomains | Where-Object {$_.IsInitial -eq $true}  | Select-Object name).name
    $TenantName=$TenantName.replace(".onmicrosoft.com","").tolower()
    $SuggestedSPNName="svc_"+$TenantName
    $SuggestedDNSName=$SuggestedSPNName+".cert."+$TenantName+".local"
    Write-host "Tenant Displayname : $AZtenantname"
    Write-host "Tenantname         : $TenantName"
    Write-host "Appreg/SPN Name    : $SuggestedSPNName"
    Write-host "DNS Name on Cert   : $SuggestedDNSName"
    Write-host
    Read-host "Press enter to continue <ENTER>"
    If ($option -ne "Option1Add")
     {
        $xSPNName=Read-host "Enter the Appreg/SPN name: [$SuggestedSPNName]"
        If ($xSPNName -eq '') 
          {
            $xSPNName=$SuggestedSPNName
          } else
          {
            $xSPNName=Remove-InvalidFileNameChars ($xSPNName)
          }     
     } else
     {
       $xSPNName=$SuggestedSPNName
     }
    $xDNSNamePrefix=Read-host "Enter the DNS prefix for the certificate: [$xSPNName]"
     If ($xDNSNamePrefix -eq '') 
      {
        $xDNSNamePrefix=$xSPNName
      } else
      {
        $xDNSNamePrefix=Remove-InvalidFileNameChars ($xDNSNamePrefix)
      }      
     $SuggestedDNSName=$xDNSNamePrefix+".cert."+$TenantName+".local"
     $xDNSName=Read-host "Full certificate DNS name will be: [$SuggestedDNSName]"
     If ($xDNSName -eq '') 
      {
        $xDNSName=$SuggestedDNSName
      } else
      {
        $xDNSName=Remove-InvalidFileNameChars ($xDNSName)
      }     
    $pass = $global:defaultpwd  # PFX Password
    $xPFXPWD=Read-host "Enter the certificate PFX Password: [$pass]"
    If ($xPFXPWD -eq '') 
      {
        $pass=$pass
      } else
      {
        $pass=$xPFXPWD
      }
    Write-host "Save password (cleartext) in export file? (Y/N) [NO]" -NoNewline
    $choice1 = (read-host).ToUpper()
    If ($null -eq $choice1) { $choice1="NO"}    
    If ($choice1 -eq "Y")
      {
        $SavePass=$true
      } else
      {
        $SavePass=$false
      }
    $currentDate = Get-Date
    Write-host "Certificate lifetime."
    Write-host "  1 - 1 month"
    Write-host "  2 - 2 months"
    Write-host "  3 - 3 months"
    Write-host "  4 - 1 year"
    Write-host "  5 - 2 years"
    Write-host "  6 - 3 years"
    Write-host "  7 - 5 years"
    Write-host "  C - Custom"
    Write-host
    $choice1 = read-host "  Enter the number [1]"
    If ($null -eq $choice1 -or "" -eq $choice1) { $choice1="1"}
    Switch ($choice1)
      {
        1 { $endDate = $currentDate.AddMonths(1) }
        2 { $endDate = $currentDate.AddMonths(2) }
        3 { $endDate = $currentDate.AddMonths(3) }
        4 { $endDate = $currentDate.AddYears(1) }
        5 { $endDate = $currentDate.AddYears(2) }
        6 { $endDate = $currentDate.AddYears(3) }
        7 { $endDate = $currentDate.AddYears(5) }
        C {
            $xAddDays = read-host "  Enter the number of days you want the certificate lifespan"
            $endDate = $currentDate.AddDays($xAddDays)
          }        
      }
    $notAfter = $endDate.AddDays(1)    
    Write-host
    Write-host "Choose where to store the created certificate:"
    Write-host "  1 - User Store"
    Write-host "  2 - Computer Store"
    Write-host
    $choice1 = read-host "  Enter the number [1]"
    If ($null -eq $choice1 -or "" -eq $choice1) { $choice1="1"}
    Switch ($choice1)
      {
        1 { $userstore=$true }
        2 { $userstore=$false }
      }    
    [hashtable]$Return = @{}
    $return.notafter=$notAfter
    $return.enddate=$endDate
    $return.currentdate=$currentDate
    $return.DNSname=$xDNSname
    $return.SPNname=$xSPNname
    $return.pass=$pass
    $return.userstore=$userstore
    $return.SavePass=$SavePass
    Return $return   
  }

Function CreateCertAndAppRegistration ($DNSName, $SPNName, $notAfter, $Pass, $CertUserStore, $addCert, $xSelectedAppID)
  {    
    $AZTenant=Get-MgOrganization 
    # Generate vars
    $rootfolder=$global:ScriptPath    
    $CertFilename=($rootfolder+$DNSName+".pfx").tolower() # Save Cert with Private key in PFX
    If ($true -eq $CertUserStore) # If Store is empty then store in user store.
      {
        $CertStore="cert:\currentuser\my\"
      }  else {
        $CertStore="cert:\LocalMachine\my\"
      }
    $thumb = (New-SelfSignedCertificate -CertStoreLocation $CertStore -DnsName $DNSName -KeyExportPolicy Exportable -Provider "Microsoft Enhanced RSA and AES Cryptographic Provider" -NotAfter $notAfter).Thumbprint
    $Pass = ConvertTo-SecureString -String $Pass -Force -AsPlainText
    Export-PfxCertificate -cert "$CertStore$thumb" -FilePath $CertFilename -Password $Pass | out-null # Out-null required, or the Return values are corrupted.
    # Load the certificate
    $cert = New-Object System.Security.Cryptography.X509Certificates.X509Certificate($CertFilename, $Pass)
    $AppID=$null
    If ($addCert -eq $true)
      {
        # $allApps=Get-MgApplication 
        # $SelectedApp=$allApps | Out-GridView -OutputMode Single
        $SelectedApp=Get-MgApplication -ApplicationId $xSelectedAppID
        $CurrentKeycreds=@()
        $CurrentKeycreds+=$($SelectedApp.KeyCredentials)
        $keyCreds = @{ 
          Type = "AsymmetricX509Cert";
          Usage = "Verify";
          key = $cert.GetRawCertData()
        }
        $CurrentKeycreds+=$keyCreds
        Update-MgApplication -ApplicationId $SelectedApp.id  -KeyCredentials $CurrentKeycreds  
        $AppId=$SelectedApp.id
      } else {
        # Create the App Reg
        $application=New-MgApplication -DisplayName $SPNname        
        $keyCreds = @{ 
          Type = "AsymmetricX509Cert";
          Usage = "Verify";
          key = $cert.GetRawCertData()
        }        
        Update-MgApplication -ApplicationId $application.id  -KeyCredentials $keyCreds  
        $AppId=$application.appid        
      }     
    $return = @{}
    $return.add('thumb',$thumb)
    $return.add('appid',$($AppID))
    $return.add('tenantid',$($AZTenant.id))
    $return.add('CertFilename',$CertFilename)    
    Return $return
  }
 
Function Option1
  {         
    Write-host
    Write-Host -ForegroundColor Green "First we need to login to Microsoft Graph with Connect-MgGraph, login in with Global Admin credentials".
    Start-Sleep 1
    Connect-MgGraph -Scopes Application.ReadWrite.All, Directory.ReadWrite.All -nowelcome
    Write-Host
    Write-Host -ForegroundColor Green "Next we ask you some information for the new Appreg/SPN and certificate details."
    Start-Sleep 1
    $AskQ=Option1AskQ       
    $returninfo=CreateCertAndAppRegistration $($askq.DNSName) $($askq.SPNName) $($askq.notAfter) $($askq.Pass) $($askq.userstore) $false  
    $AuthFilename=$($ReturnInfo.certfilename).Replace(".pfx",".txt").ToLower() # Save all info to this file.    
    If ($AskQ.SavePass)
      {
        $displaypass=$($askq.Pass)
      } else
      {
        $displaypass="**********"
      }
    Write-Host
    Write-host "Summary:"
    Write-host "SPN Name              : $($askq.SPNName)"
    Write-host "DNS Name on Cert      : $($askq.DNSName)"
    Write-host "PFX password          : $displaypass"
    Write-host "Authentication file   : $AuthFilename"
    Write-host "Certificate PFX file  : $($ReturnInfo.certfilename)"
    Write-host "Cert date currentdate : $($askq.currentdate)"
    Write-host "Cert date endDate     : $($askq.enddate)"
    Write-host "Cert date notAfter    : $($askq.notAfter)"      
    Read-host "Press Enter to continue"
    #$AuthFilename="C:\OneDrive\SCRIPTS\VSCode.tests\test.txt"
    $AuthFileContent=@()
    $AuthFileContent+="Entra ID - App Registrion information"
    $AuthFileContent+="#####################################"
    $AuthFileContent+="Tenant ID     : "+$($ReturnInfo.TenantID)
    $AuthFileContent+="Application ID: "+$($ReturnInfo.appid)
    $AuthFileContent+="Thumbprint    : "+$($ReturnInfo.thumb)
    If ($AskQ.SavePass)
      {
        $AuthFileContent+="PFX Password  : "+$($Askq.Pass)
      }
    $AuthFileContent+=""
    $AuthFileContent+="# PowerShell Code"
    $AuthFileContent+="$"+"tenantid='$($ReturnInfo.TenantID)'"
    $AuthFileContent+="$"+"appid='$($ReturnInfo.appid)'"
    $AuthFileContent+="$"+"thumb='$($ReturnInfo.thumb)'"
    $AuthFileContent+=""
    $AuthFileContent+="# Example how to connect to PowerShell Az"
    $AuthFileContent+="# Import-Module Az"
    $AuthFileContent+="# Add-AzAccount -TenantId $"+"tenantid -ApplicationId $"+"appid -CertificateThumbprint $"+"thumb"
    $AuthFileContent+=""
    $AuthFileContent+="# Example how to connect to PowerShell Microsoft Graph"
    $AuthFileContent+="# Import-Module Microsoft.Graph"      
    $AuthFileContent+="# Connect-MgGraph -TenantId $"+"tenantid -ApplicationId $"+"appid -CertificateThumbprint $"+"thumb"
    $AuthFileContent+=""
    $AuthFileContent+="XML Config entries"
    $AuthFileContent+="<TenantID>$($ReturnInfo.TenantID)</TenantID>"
    $AuthFileContent+="<AppID>$($ReturnInfo.appid)</AppID>"
    $AuthFileContent+="<Thumb>$($ReturnInfo.thumb)</Thumb>"
    $AuthFileContent | out-file $AuthFilename -Force         
    Write-host
    Write-host "Information written to file: $AuthFilename"
    Write-host "Done."
    ExitScript
  } #/Option1

  Function Option1Add
  {         
    Write-host
    Write-Host -ForegroundColor Green "First we need to login to Microsoft Graph with Connect-MGgraph, login in with Global Admin credentials".
    Start-Sleep 1
    Connect-MgGraph -Scopes Application.ReadWrite.All, Directory.ReadWrite.All -nowelcome
    Write-Host "--> Next we ask you to select the current App Registration where you want to add the new certificate."
    start-sleep 2    
    $allApps=Get-MgApplication
    $SelectedApp=$allApps | Out-GridView -OutputMode Single -title "Select the AppReg where you want to add a new certificate."
    Write-Host -ForegroundColor Green "Next we ask you some information for the new certificate, after the creation of the cert you can select the Appreg/SPN to assign it to."
    Start-Sleep 1
    $AskQ=Option1AskQ Option1Add
    $returninfo=CreateCertAndAppRegistration $($askq.DNSName) $($askq.SPNName) $($askq.notAfter) $($askq.Pass) $($askq.userstore) $true $($selectedApp.id)
    $AuthFilename=$($ReturnInfo.certfilename).Replace(".pfx",".txt").ToLower() # Save all info to this file.
    If ($AskQ.SavePass)
      {
        $displaypass=$($askq.Pass)
      } else
      {
        $displaypass="**********"
      }
    Write-Host
    Write-host "Certificate assigned to Appreg/SPN:"
    Write-host "SPN Name              : $($askq.SPNName)"
    Write-host "DNS Name on Cert      : $($askq.DNSName)"
    Write-host "PFX password          : $displaypass"
    Write-host "Authentication file   : $AuthFilename"
    Write-host "Certificate PFX file  : $($ReturnInfo.certfilename)"
    Write-host "Cert date currentdate : $($askq.currentdate)"
    Write-host "Cert date endDate     : $($askq.enddate)"
    Write-host "Cert date notAfter    : $($askq.notAfter)"      
    Read-host "Press Enter to continue"    
    $AuthFileContent=@()
    $AuthFileContent+="Entra ID - App Registrion information"
    $AuthFileContent+="#####################################"
    $AuthFileContent+="Tenant ID     : "+$($ReturnInfo.TenantID)
    $AuthFileContent+="Application ID: "+$($ReturnInfo.appid)
    $AuthFileContent+="Thumbprint    : "+$($ReturnInfo.thumb)
    If ($AskQ.SavePass)
      {
        $AuthFileContent+="PFX Password  : "+$($Askq.SavePass)
      }
    $AuthFileContent+=""
    $AuthFileContent+="# PowerShell Code"
    $AuthFileContent+="$"+"tenantid='$($ReturnInfo.TenantID)'"
    $AuthFileContent+="$"+"appid='$($ReturnInfo.appid)'"
    $AuthFileContent+="$"+"thumb='$($ReturnInfo.thumb)'"
    $AuthFileContent+=""
    $AuthFileContent+="# Example how to connect to PowerShell Az"
    $AuthFileContent+="# Import-Module Az"
    $AuthFileContent+="# Add-AzAccount -TenantId $"+"tenantid -ApplicationId $"+"appid -CertificateThumbprint $"+"thumb"
    $AuthFileContent+=""
    $AuthFileContent+="# Example how to connect to PowerShell Microsoft Graph"
    $AuthFileContent+="# Import-Module Microsoft.Graph"      
    $AuthFileContent+="# Connect-MgGraph -TenantId $"+"tenantid -ApplicationId $"+"appid -CertificateThumbprint $"+"thumb"
    $AuthFileContent+=""
    $AuthFileContent+="XML Config entries"
    $AuthFileContent+="<TenantID>$($ReturnInfo.TenantID)</TenantID>"
    $AuthFileContent+="<AppID>$($ReturnInfo.appid)</AppID>"
    $AuthFileContent+="<Thumb>$($ReturnInfo.thumb)</Thumb>"
    $AuthFileContent | out-file $AuthFilename -Force         
    Write-host
    Write-host "Information written to file: $AuthFilename"
    Write-host "Done."
    ExitScript
  } #/Option1Add

  Function DeleteAppRegs
  {
    Connect-MgGraph -Scopes Application.ReadWrite.All, Directory.ReadWrite.All -nowelcome
    $AllAppRegs=Get-MgApplication
    $AllSelectedAppRegs=$AllAppRegs | Out-GridView -OutputMode Multiple
    Write-host -ForegroundColor Yellow "These are the selected AppRegs:"
    $AllSelectedAppRegs.DisplayName
    Write-host
    Write-host -ForegroundColor green "Are you sure you want to delete these Application Registrations from Entra ID? (YES/NO): " -NoNewline    
    $choice1 = (read-host).ToUpper()
    If ($choice1 -eq "YES")
      {
        Write-host "Yes deleting..."
        Foreach ($SelectedAppReg in $AllSelectedAppRegs)
          {
            Write-host -ForegroundColor DarkYellow  "Removing: $($SelectedAppReg.DisplayName) ..." -NoNewline
            Remove-MgApplication -ApplicationId $($SelectedAppReg.id) -Confirm:$false #-WhatIf
            Write-host -ForegroundColor Green "done!"
          }          
      } else
      {
        Write-host "Nothing todo..."
      }
      ExitScript
  } #/DeleteAppRegs

Function AppRegOverview
  {
    $c=";"
    $Now = Get-Date
    Connect-MgGraph -Scopes Application.ReadWrite.All -nowelcome
    $AZTenant=Get-MgOrganization
    $AZtenantname=Remove-InvalidFileNameChars($($AZTenant.DisplayName))
    $AZTenantID=$($aztenant.id)    
    $AllApps=Get-MgApplication -All
    $a=New-Object System.Collections.ArrayList 
    $logheader="timestamp"+$c+"TenantName"+$c+"TenantID"+$c+"Appname"+$c+"ObjectID"+$c+"AppID"+$c+"Cert/Secret"+$c+"Cert/Secret-Name"+$c+"StartDate"+$c+"Enddate"+$c+"RemainingDays"+$c+"Warning"
    $a.add($logheader) | out-null
    Write-host -ForegroundColor green $logheader
    $color="green"    
    Foreach ($app in $allapps)
      {
        $AppName = $App.DisplayName
        $AppID   = $App.Id # (object ID)
        $ApplID  = $App.AppId
        $Secrets = $App.PasswordCredentials
        $Certs   = $App.KeyCredentials
        foreach ($Cert in $Certs) {
          $StartDate = $Cert.StartDateTime
          $EndDate   = $Cert.EndDateTime
          $CertName  = $Cert.DisplayName
          $RemainingDaysCount = ($EndDate - $Now).Days
          If ($RemainingDaysCount -le 0)
            {
              $warning="EXPIRED"
              $color="red"
            } elseif ($RemainingDaysCount -le $expirewarning) 
            {
              $warning="EXPIRES SOON"
              $color="yellow"
            } else {
              $warning="-"
            }

          $logline=$($global:timestamp)+$c+$AZtenantname+$c+$AZtenantID+$c+$AppName+$c+$AppID+$c+$ApplID+$c+"Certificate"+$c+$certname+$c+$StartDate+$c+$EndDate+$c+$RemainingDaysCount+$c+$warning
          Write-host -ForegroundColor $color $logline          
          $a.add($logline) | out-null
        }
        foreach ($Secret in $Secrets) {
          $StartDate  = $Secret.StartDateTime
          $EndDate    = $Secret.EndDateTime
          $SecretName = $Secret.DisplayName
          $RemainingDaysCount = ($EndDate - $Now).Days 
          If ($RemainingDaysCount -le 0)
            {
              $warning="EXPIRED"
              $color="red"
            } elseif ($RemainingDaysCount -le $expirewarning) 
            {
              $warning="EXPIRES SOON"
              $color="yellow"
            } else {
              $warning="-"
            }         
          $logline=$($global:timestamp)+$c+$AZtenantname+$c+$AZtenantID+$c+$AppName+$c+$AppID+$c+$ApplID+$c+"Secret"+$c+$SecretName+$c+$StartDate+$c+$EndDate+$c+$RemainingDaysCount+$c+$warning          
          Write-host -ForegroundColor $color $logline          
          $a.add($logline) | out-null
        }
      }
    Write-host "Want to save the output to a CSV file? (Y/N) [Y]: " -NoNewline
    $choice1 = (read-host).ToUpper()    
    If ("" -eq $choice1) { $choice1="Y"}    
    If ($choice1 -eq "Y")
      {
        $fntimestamp=(Get-Date).ToString("yyyyMMdd-HHmm")
        $fnsuggestion=$fntimestamp+"_appreg_expiration_overview.csv"
        Write-host "Filename [$fnsuggestion]:" -NoNewline
        $fnanswer=read-host
        If ("" -eq $fnanswer)
          {
            $fn=$($global:ScriptPath)+$fnsuggestion            
            $a | out-file -FilePath $fn
            write-host "Output saved to file: $fn"
          } else
          {
            $fn=$($global:ScriptPath)+$fnanswer            
            $a | out-file -FilePath $fn
            write-host "Output saved to file: $fn"
          }
      } else
      {
        # no
        Write-host "No don't save."
      }    
    ExitScript
  } #/AppRegOverview

Function AppRegPermissions
  {
    Connect-MgGraph -Scopes Application.ReadWrite.All -nowelcome
    $AZTenant=Get-MgOrganization
    $AZtenantname=Remove-InvalidFileNameChars($($AZTenant.DisplayName))
    $AZTenantID=$($aztenant.id)     
    $APIPermissions=Find-MgGraphPermission -All
    $Applications=Get-MgApplication -All
    $AllDirectoryRoles=Get-MgDirectoryRole -All
    $b=New-Object System.Collections.ArrayList 
    $c=";"
    $logheader="timestamp"+$c+"TenantName"+$c+"TenantID"+$c+"Appname"+$c+"ObjectID"+$c+"AppID"+$c+"DirectoryRole/APIPermission"+$c+"Role/Permissions"
    Write-host -ForegroundColor green $logheader
    $b.add($logheader) | out-null
    #Initialize array
    $global:Permissionstable = @()    
    foreach ($Application in $Applications) 
    {
      $AppName = $Application.DisplayName
      $AppID   = $Application.Id # (object ID)
      $ApplID  = $Application.AppId
      Foreach ($Role in $AllDirectoryRoles)
      {
        $AllRoleMembers=Get-MgDirectoryRoleMemberAsServicePrincipal -DirectoryRoleId $($role.id)
        Foreach ($rolemember in $AllRoleMembers)
        {
          If ($($Application.appid) -eq $($rolemember.appid))
            {
              $logline=$($global:timestamp)+$c+$AZtenantname+$c+$AZtenantID+$c+$AppName+$c+$AppID+$c+$ApplID+$c+"DirectoryRole"+$c+$($role.displayname)
              Write-host -ForegroundColor yellow $logline
              $b.add($logline) | out-null
            }          
        }      
      }   
      foreach ($permission in $Application.requiredResourceAccess.resourceAccess) 
      {  
          $APIPermissions | ForEach-Object {
              if ($permission.id -eq $_.id) {
                  $logline=$($global:timestamp)+$c+$AZtenantname+$c+$AZtenantID+$c+$AppName+$c+$AppID+$c+$ApplID+$c+"Graph Permissions"+$c+$($($_.name))
                  Write-host -ForegroundColor yellow $logline
                  $b.add($logline) | out-null
                  $row = new-object PSObject -Property @{
                      ApplicationName = $($Application.displayName);
                      RoleName        = $($_.name)
                  }  
                  $global:Permissionstable += $row
              }
          }
      }
    }
  Write-host "Want to save the output to a CSV file? (Y/N) [Y]: " -NoNewline
  $choice1 = (read-host).ToUpper()    
  If ("" -eq $choice1) { $choice1="Y"}    
  If ($choice1 -eq "Y")
    {
      $fntimestamp=(Get-Date).ToString("yyyyMMdd-HHmm")
      $fnsuggestion=$fntimestamp+"_appreg_permissions_overview.csv"
      Write-host "Filename [$fnsuggestion]:" -NoNewline
      $fnanswer=read-host
      If ("" -eq $fnanswer)
        {
          $fn=$($global:ScriptPath)+$fnsuggestion            
          $b | out-file -FilePath $fn
          write-host "Output saved to file: $fn"
        } else
        {
          $fn=$($global:ScriptPath)+$fnanswer            
          $b | out-file -FilePath $fn
          write-host "Output saved to file: $fn"
        }
    } else
    {      
      Write-host "No don't save."
    }
    ExitScript
  } #/AppReg-Permissions
Clear-Host
Write-host -ForegroundColor green "Welcome at Entra ID - AppReg Manager 2024 - v03"
Write-host
Write-host -ForegroundColor white "1 - Connect to MgGraph and Create AppReg/SPN with self signed certificate"
Write-host -ForegroundColor white "2 - Connect to MgGraph and Delete one or more AppReg/SPN"
Write-host -ForegroundColor white "3 - Connect to MgGraph and Check expired certificates or secrets"
Write-host -ForegroundColor white "4 - Connect to MgGraph and Check AppReg/SPN permissions"
Write-host -ForegroundColor white "5 - Connect to MgGraph and Update AppReg/SPN certificate"
Write-host -ForegroundColor white "Q - Quit"
Write-host
$choice1 = read-host "  Enter the number"
Switch ($choice1)
  {
    1 { Option1 }
    2 { DeleteAppRegs }
    3 { AppRegOverview }
    4 { AppRegPermissions }
    5 { Option1Add }
    Q { exitscript }
  }

